//---------------------------------------------------------------------------
/*
 * Copyright (c) Allan Petersen, 2001.
 * This is a tutorial for a more advanced directory monitoring system
 * based on a multithreaded monitor who transmits the update message
 * through the Windows message queue.
 *
 * (c) Copyright 2001, Allan Petersen
 * ALL RIGHTS RESERVED
 * Permission to use, copy, modify, and distribute this software for
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that
 * the name of Allan Petersen not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission.
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL ALLAN
 * PETERSEN BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT ALLAN PETERSEN HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Contact Allan Petersen at <support@allanpetersen.com> or visit
 * www.allanpetersen.com
 *
 */
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "AdvDirMonitorThread.h"
//---------------------------------------------------------------------------

__fastcall TDirMonitorThread::TDirMonitorThread(bool CreateSuspended,
	TControl *ATargetControl) : TThread(CreateSuspended)
{
	FreeOnTerminate = true;

    Priority = tpNormal;
    
	FTargetControl = ATargetControl;
    FWatchPath  = "";
	OnTerminate = DirMonitorTerminate;
}
//---------------------------------------------------------------------------
void __fastcall TDirMonitorThread::DirMonitorTerminate(TObject *Sender)
{
}
//---------------------------------------------------------------------------
void __fastcall TDirMonitorThread::SetWatchPath(AnsiString Value)
{

    if (FWatchPath != Value) {
     	FWatchPath = Value;
        }
}
//---------------------------------------------------------------------------
AnsiString   __fastcall TDirMonitorThread::GetChanges (void)
{
    return (ChangeString);
}
//---------------------------------------------------------------------------
void __fastcall TDirMonitorThread::Execute()
{
    FILE_NOTIFY_INFORMATION *pNotify;
    OVERLAPPED              *pOverlapped;
    HANDLE                   hDirectory;
    DWORD                    BytesReturned;

	//---- Place thread code here ----
    pOverlapped = new OVERLAPPED;
    memset (pOverlapped, NULL, sizeof (OVERLAPPED));
    pNotify     = new FILE_NOTIFY_INFORMATION[10];
    memset (pNotify, NULL, sizeof (FILE_NOTIFY_INFORMATION[10]));

    hDirectory = CreateFile (FWatchPath.c_str(),
                              GENERIC_READ,
                              FILE_SHARE_READ|FILE_SHARE_DELETE,
                              NULL,
                              OPEN_EXISTING,
                              FILE_FLAG_BACKUP_SEMANTICS | FILE_FLAG_OVERLAPPED,
                              NULL
                          );
    while (!Terminated) {
        if (hDirectory != INVALID_HANDLE_VALUE) {
            pOverlapped->hEvent = NULL;
            if (ReadDirectoryChangesW (hDirectory,
                                        pNotify,	// pointer to the buffer to receive the read results
                                        sizeof(FILE_NOTIFY_INFORMATION)*9,	// length of lpBuffer
                                        true,	// flag for monitoring directory or directory tree
                                        FILE_NOTIFY_CHANGE_LAST_WRITE |
                                        FILE_NOTIFY_CHANGE_FILE_NAME   |
                                        FILE_NOTIFY_CHANGE_LAST_ACCESS |
                                        FILE_NOTIFY_CHANGE_CREATION	,	// filter conditions to watch for
                                        &BytesReturned,	// number of bytes returned
                                        pOverlapped,	// pointer to structure needed for overlapped I/O
                                        NULL// pointer to completion routine
                                       )){
                BytesReturned = 0;
                while (!Terminated && BytesReturned == 0) {
                    GetOverlappedResult(hDirectory, pOverlapped, &BytesReturned, false);
                    }
                switch (pNotify[0].Action) {
                    case FILE_ACTION_ADDED:
                        pNotify[0].FileName[pNotify[0].FileNameLength/sizeof (wchar_t)] = NULL;
                        ChangeString = "File Added:" + String(pNotify[0].FileName);
                        Synchronize (UpdateDirectory);
                        break;
                    case FILE_ACTION_REMOVED:
                        pNotify[0].FileName[pNotify[0].FileNameLength/sizeof (wchar_t)] = NULL;
                        ChangeString = "File Removed:" + String(pNotify[0].FileName);
                        Synchronize (UpdateDirectory);
                        break;
                    case FILE_ACTION_MODIFIED:
                        pNotify[0].FileName[pNotify[0].FileNameLength/sizeof (wchar_t)] = NULL;
                        ChangeString = "File Modified: " + String(pNotify[0].FileName);
                        Synchronize (UpdateDirectory);
                        break;
                    case FILE_ACTION_RENAMED_OLD_NAME:
                        pNotify[0].FileName[pNotify[0].FileNameLength/sizeof (wchar_t)] = NULL;
                        ChangeString = "File Renamed (old):" + String(pNotify[0].FileName);
                        Synchronize (UpdateDirectory);
                        break;
                    case FILE_ACTION_RENAMED_NEW_NAME:
                        pNotify[0].FileName[pNotify[0].FileNameLength/sizeof (wchar_t)] = NULL;
                        ChangeString = "File Renamed (new):" + String(pNotify[0].FileName);
                        Synchronize (UpdateDirectory);
                        break;
                        }
                }
            }
        }
    if (hDirectory != NULL)
        CloseHandle (hDirectory);
    if (pOverlapped)
        delete pOverlapped;
    if (pNotify)
        delete[] pNotify;
}
//---------------------------------------------------------------------------
void __fastcall TDirMonitorThread::UpdateDirectory()
{
	if (FTargetControl) {
        // Send the notification through the Windows message queue
    	FTargetControl->Perform(CM_DIRCONTENTSCHANGED, 0, 0);
        }
}
//---------------------------------------------------------------------------


