//---------------------------------------------------------------------------
/*
 * Copyright (c) Allan Petersen, 2001.
 * This is a tutorial for a more advanced directory monitoring system
 * based on a multithreaded monitor who transmits the update message
 * through the Windows message queue.
 *
 * (c) Copyright 2001, Allan Petersen
 * ALL RIGHTS RESERVED
 * Permission to use, copy, modify, and distribute this software for
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that
 * the name of Allan Petersen not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission.
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL ALLAN
 * PETERSEN BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT ALLAN PETERSEN HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED ANDnnn66 ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Contact Allan Petersen at <support@allanpetersen.com> or visit
 * www.allanpetersen.com
 *
 */
//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "AdvDirMonitorThread.h"
#include "UMain.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
        : TForm(Owner)
{
    StatusBar1->SimpleText = "Monitoring: " + FileListBox1->Directory;

    // create a new directory monitor thread, initially
    // suspended with Form1 (this) as the TargetControl
    pDirMonitorThread = new TDirMonitorThread(true, this);

    // set the thread to monitor the FileListBox's Directory
    pDirMonitorThread->SetWatchPath(FileListBox1->Directory);

    // Start the process
    pDirMonitorThread->Resume();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::CMDirContentsChanged(TMessage &Msg)
{
	// the DirMonitorThread has sent the
    // CM_DIRCONTENTSCHANGED message -- refresh the
    // contents of the FileListBox from disk
	FileListBox1->Update();

    Memo1->Lines->Add("Directory Changed, Time : " + Now().TimeString());
    // ---- Display the message
    Memo1->Lines->Add(pDirMonitorThread->GetChanges ());
}
//---------------------------------------------------------------------------
void __fastcall TForm1::DirectoryListBox1Change(TObject *Sender)
{
    // Terminate current running thread
	pDirMonitorThread->Terminate();

    // wait for the thread to terminate, it can take a few seconds
    // to terminate it, but we don't care cause we don't need to
    // use if after termination
    pDirMonitorThread->WaitFor ();


    StatusBar1->SimpleText = "Monitoring: " + FileListBox1->Directory;

    // create a new directory monitor thread that is suspended
    pDirMonitorThread = new TDirMonitorThread(true, this);

    // set the thread to monitor the FileListBox's Directory
    pDirMonitorThread->SetWatchPath(FileListBox1->Directory);

    // Start the thread
    pDirMonitorThread->Resume();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormDestroy(TObject *Sender)
{
	// terminate the thread
	pDirMonitorThread->Terminate ();
    pDirMonitorThread->WaitFor ();
    pDirMonitorThread = NULL;
}
//---------------------------------------------------------------------------

